@@ -54,7 +54,11 @@ ci_upper <- function(mean, ssd, n, conf = 0.95) {
 #' ci_var(dft, Fare, Pclass, conf = 0.99)
 #' @export
 ci_var <- function(df, var, group_var = NULL, conf = 0.95) {
-  var <- enquo(var)
+  if (isTRUE(try(is.character(var), silent = TRUE))) {
+    var <- eval(substitute(var), df)
+  } else {
+    var <- as_label(enquo(var))
+  }
   group_var <- enquo(group_var)
 
   if (as_label(group_var) != "NULL") {
@@ -73,6 +77,7 @@ ci_var <- function(df, var, group_var = NULL, conf = 0.95) {
     )
 
   varname <- as_label(var)
+  varname <- gsub('"', "", varname)
   cols <- colnames(aux)
   colnames(aux)[cols == "smean"] <- sprintf("%s_mean", varname)
   colnames(aux)[cols == "ssd"] <- sprintf("%s_sd", varname)
